"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactQueryVisitor = void 0;
const auto_bind_1 = __importDefault(require("auto-bind"));
const change_case_all_1 = require("change-case-all");
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const fetcher_custom_mapper_1 = require("./fetcher-custom-mapper");
const variables_generator_1 = require("./variables-generator");
class ReactQueryVisitor extends visitor_plugin_common_1.ClientSideBaseVisitor {
    constructor(schema, fragments, rawConfig, documents) {
        super(schema, fragments, rawConfig, {
            documentMode: visitor_plugin_common_1.DocumentMode.string,
            errorType: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.errorType, 'unknown'),
            exposeDocument: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.exposeDocument, false),
            exposeQueryKeys: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.exposeQueryKeys, false),
            exposeQueryClientHook: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.exposeQueryClientHook, false),
            exposeMutationKeys: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.exposeMutationKeys, false),
            exposeFetcher: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.exposeFetcher, false),
            addInfiniteQuery: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.addInfiniteQuery, false),
            legacyMode: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.legacyMode, false),
        });
        this.rawConfig = rawConfig;
        this.reactQueryHookIdentifiersInUse = new Set();
        this.reactQueryOptionsIdentifiersInUse = new Set();
        this.queryMethodMap = {
            infiniteQuery: {
                hook: 'useInfiniteQuery',
                options: 'UseInfiniteQueryOptions',
            },
            query: {
                hook: 'useQuery',
                options: 'UseQueryOptions',
            },
            mutation: {
                hook: 'useMutation',
                options: 'UseMutationOptions',
            },
            subscription: {
                hook: 'useSubscription',
                options: 'UseSubscriptionOptions',
            },
        };
        this._externalImportPrefix = this.config.importOperationTypesFrom
            ? `${this.config.importOperationTypesFrom}.`
            : '';
        this._documents = documents;
        this.fetcher = this.createFetcher(rawConfig.fetcher || 'fetch');
        (0, auto_bind_1.default)(this);
    }
    get imports() {
        return this._imports;
    }
    createFetcher(raw) {
        // @TODO: imagine10255 remove other method
        // if (typeof raw === 'object' && 'endpoint' in raw) {
        //     return new HardcodedFetchFetcher(this, raw);
        // }
        return new fetcher_custom_mapper_1.CustomMapperFetcher(this, raw);
    }
    get hasOperations() {
        return this._collectedOperations.length > 0;
    }
    getImports() {
        const baseImports = super.getImports();
        if (!this.hasOperations) {
            return baseImports;
        }
        const hookAndTypeImports = [
            ...Array.from(this.reactQueryHookIdentifiersInUse),
            'useQueryClient',
            'QueryClient',
            'Updater',
            ...Array.from(this.reactQueryOptionsIdentifiersInUse).map(identifier => `${this.config.useTypeImports ? 'type ' : ''}${identifier}`),
        ];
        const moduleName = this.config.legacyMode ? 'react-query' : '@tanstack/react-query';
        return [
            ...baseImports,
            `import { ${hookAndTypeImports.join(', ')} } from '${moduleName}';`,
            `import {gql, useSubscription, SubscriptionHookOptions} from '@apollo/client';`,
        ];
    }
    getFetcherImplementation() {
        return this.fetcher.generateFetcherImplementaion();
    }
    _getHookSuffix(name, operationType) {
        if (this.config.omitOperationSuffix) {
            return '';
        }
        if (!this.config.dedupeOperationSuffix) {
            return (0, change_case_all_1.pascalCase)(operationType);
        }
        if (name.includes('Query') || name.includes('Mutation') || name.includes('Subscription')) {
            return '';
        }
        return (0, change_case_all_1.pascalCase)(operationType);
    }
    buildOperation(node, documentVariableName, operationType, operationResultType, operationVariablesTypes, hasRequiredVariables) {
        var _a, _b;
        const nodeName = (_b = (_a = node.name) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : '';
        // const suffix = this._getHookSuffix(nodeName, operationType);
        const operationName = this.convertName(nodeName, {
            // suffix,
            useTypesPrefix: false,
            useTypesSuffix: false,
        });
        operationResultType = this._externalImportPrefix + operationResultType;
        operationVariablesTypes = this._externalImportPrefix + operationVariablesTypes;
        let query = null;
        switch (operationType) {
            case 'Query':
                query = this.fetcher.generateQueryHook(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables);
                if (this.config.exposeDocument) {
                    query += `\nuse${operationName}.document = ${documentVariableName};\n`;
                }
                if (this.config.exposeQueryKeys) {
                    query += `\n${(0, variables_generator_1.generateQueryKeyMaker)(node, operationName, operationVariablesTypes, hasRequiredVariables)}\n`;
                }
                if (this.config.exposeQueryClientHook) {
                    query += `\n${(0, variables_generator_1.generateQueryClickHook)(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables)}\n`;
                }
                if (this.config.addInfiniteQuery) {
                    query += `\n${this.fetcher.generateInfiniteQueryHook(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables)}\n`;
                    if (this.config.exposeQueryKeys) {
                        query += `\n${(0, variables_generator_1.generateInfiniteQueryKeyMaker)(node, operationName, operationVariablesTypes, hasRequiredVariables)}\n`;
                    }
                }
                // The reason we're looking at the private field of the CustomMapperFetcher to see if it's a react hook
                // is to prevent calling generateFetcherFetch for each query since all the queries won't be able to generate
                // a fetcher field anyways.
                if (this.config.exposeFetcher && !this.fetcher._isReactHook) {
                    query += this.fetcher.generateFetcherFetch(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables);
                }
                return query;
            case 'Mutation':
                query = this.fetcher.generateMutationHook(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables);
                if (this.config.exposeMutationKeys) {
                    query += (0, variables_generator_1.generateMutationKeyMaker)(node, operationName);
                }
                if (this.config.exposeFetcher && !this.fetcher._isReactHook) {
                    query += this.fetcher.generateFetcherFetch(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables);
                }
                return query;
            case 'Subscription':
                // console.warn(
                //     `Plugin "typescript-react-query" does not support GraphQL Subscriptions at the moment! Ignoring "${node.name.value}"...`,
                // );
                query = this.fetcher.generateSubscriptionHook(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables);
                if (this.config.exposeMutationKeys) {
                    query += (0, variables_generator_1.generateMutationKeyMaker)(node, operationName);
                }
                if (this.config.exposeFetcher && !this.fetcher._isReactHook) {
                    query += this.fetcher.generateFetcherFetch(node, documentVariableName, operationName, operationResultType, operationVariablesTypes, hasRequiredVariables);
                }
                return query;
        }
        return null;
    }
}
exports.ReactQueryVisitor = ReactQueryVisitor;
//# sourceMappingURL=visitor.js.map